@@ -31,6 +31,7 @@ import math
 import random
 import time
 import warnings
+import weakref
 
 import pygame
 import six
@@ -41,6 +42,9 @@ import sge
 # How often to call cache.prune in milliseconds.
 CACHE_PRUNE_TIME = 8000
 
+# How long cached items should remain cached by default in seconds.
+CACHE_DEFAULT_LIFE = 15
+
 # Lists of objects that are tangible and objects that check for
 # collisions; makes collision detection more efficient.
 _colliders = []
@@ -62,7 +66,7 @@ class cache(object):
     _prune = {}
 
     @classmethod
-    def add(cls, i, value, prune_time=30):
+    def add(cls, i, value, prune_time=CACHE_DEFAULT_LIFE):
         # Add value with index ``i`` to cache as ``value``.
         # Automatically deleted after ``prune_time`` seconds.
         cls._cache[i] = value
@@ -1068,8 +1072,9 @@ def s_get_image(self, num, xscale=1, yscale=1, rotation=0, alpha=255,
     if isinstance(self, sge.gfx.Sprite):
         num %= self.frames
 
-        i = ("s_image", self, self.rd["drawcycle"], num, xscale, yscale,
-             rotation, alpha, tuple(blend) if blend is not None else None)
+        i = ("s_image", weakref.ref(self), self.rd["drawcycle"], num, xscale,
+             yscale, rotation, alpha,
+             tuple(blend) if blend is not None else None)
         img = cache.get(i)
         if img is None:
             if xscale != 0 and yscale != 0:
@@ -1107,8 +1112,8 @@ def s_get_image(self, num, xscale=1, yscale=1, rotation=0, alpha=255,
 def s_get_precise_mask(self, num, xscale, yscale, rotation):
     # Return a precise mask (2D list of True/False values) for the
     # given image index.
-    i = ("s_mask", self, self.width, self.height, self.rd["drawcycle"],
-         num, xscale, yscale, rotation)
+    i = ("s_mask", weakref.ref(self), self.width, self.height,
+         self.rd["drawcycle"], num, xscale, yscale, rotation)
     mask = cache.get(i)
     if mask is None:
         image = s_set_transparency(self, self.rd["baseimages"][num])
@@ -1140,6 +1145,28 @@ def s_get_precise_mask(self, num, xscale, yscale, rotation):
     return mask
 
 
+def s_from_text(cls, font, text, width, height, color, halign, valign,
+                anti_alias):
+    # Version of gfx.Sprite.from_text which always returns the same
+    # sprite, to avoid unnecessary duplication for things like
+    # projections.
+    f_name = tuple(font.name) if font.name is not None else None
+    i = ("text_sprite", cls, f_name, font.size, font.underline, font.bold,
+         font.italic, text, width, height, str(color), halign, valign,
+         anti_alias)
+    s = cache.get(i)
+    if s is None:
+        w = font.get_width(text, width, height)
+        h = font.get_height(text, width, height)
+        x = {"left": 0, "right": w, "center": w / 2}.get(halign.lower(), 0)
+        y = {"top": 0, "bottom": h, "middle": h / 2}.get(valign.lower(), 0)
+        s = cls(width=w, height=h, origin_x=x, origin_y=y)
+        s.draw_text(font, text, x, y, width, height, color, halign, valign)
+
+    cache.add(i, s)
+    return s
+
+
 def tg_blit(self, dest, x, y):
     # Blit the tile grid onto a Pygame surface.
     # Note: origin is NOT taken into account here!
